@@ -6,7 +6,7 @@
 
 void GitBlameTest::testBlameFiles()
 {
-    QDir rootDir(":/");
+    QDir rootDir(QStringLiteral(":/"));
     const QFileInfoList files = rootDir.entryInfoList(QDir::Files);
     for (const auto &fileInfo : files) {
         qDebug() << fileInfo.absoluteFilePath();
@@ -20,3 +20,5 @@ void GitBlameTest::testBlameFiles()
 }
 
 QTEST_MAIN(GitBlameTest)
+
+#include "moc_gitblametest.cpp"
