@@ -136,7 +136,7 @@ public:
     FileLineReader(const QUrl &url)
         : file(url.toLocalFile())
     {
-        file.open(QIODevice::ReadOnly);
+        std::ignore = file.open(QIODevice::ReadOnly);
     }
 
     // called with non-descending lineno
@@ -378,6 +378,7 @@ class LSPClientPluginViewImpl : public QObject, public KXMLGUIClient
     QPointer<QAction> m_diagnostics;
     QPointer<QAction> m_messages;
     QPointer<QAction> m_closeDynamic;
+    QPointer<QAction> m_suspendAll;
     QPointer<QAction> m_restartServer;
     QPointer<QAction> m_restartAll;
     QPointer<QAction> m_switchSourceHeader;
@@ -612,6 +613,11 @@ public:
         m_restartAll = actionCollection()->addAction(QStringLiteral("lspclient_restart_all"), this, &self_type::restartAll);
         m_restartAll->setText(i18n("Restart All LSP Servers"));
 
+        // suspend all - should shut down all servers and prohibit new ones from starting till toggled
+        m_suspendAll = actionCollection()->addAction(QStringLiteral("lspclient_suspend_all"), this, &self_type::suspendAll);
+        m_suspendAll->setText(i18n("Suspend All"));
+        m_suspendAll->setCheckable(true);
+
         auto *goToAction = new QAction(i18n("Go To"));
         actionCollection()->addAction(QStringLiteral("lspclient_goto_menu"), goToAction);
         auto *goTo = new QMenu();
@@ -630,6 +636,7 @@ public:
         lspOther->addAction(m_triggerSymbolInfo);
         lspOther->addSeparator();
         lspOther->addAction(m_closeDynamic);
+        lspOther->addAction(m_suspendAll);
         lspOther->addSeparator();
         lspOther->addAction(m_restartServer);
         lspOther->addAction(m_restartAll);
@@ -663,7 +670,7 @@ public:
 
         connect(m_mainWindow, &KTextEditor::MainWindow::viewCreated, this, &self_type::onViewCreated);
 
-        connect(this, &self_type::ctrlClickDefRecieved, this, &self_type::onCtrlMouseMove);
+        connect(this, &self_type::ctrlClickDefReceieved, this, &self_type::onCtrlMouseMove);
 
         configUpdated();
 
@@ -1192,6 +1199,18 @@ public:
         }
     }
 
+    void suspendAll()
+    {
+        if (m_suspendAll->isChecked()) {
+            m_serverManager->setSuspendEnabled(true);
+        } else {
+            m_serverManager->setSuspendEnabled(false);
+        }
+
+        // If suspend is enabled then this will just stop the servers otherwise properly restart them
+        m_serverManager->restart(nullptr);
+    }
+
     // local helper to overcome some differences in LSP types
     struct RangeItem {
         QUrl uri;
@@ -1228,10 +1247,10 @@ public:
                 return QStandardItem::data(role);
             }
 
-            auto line = data(Qt::UserRole);
+            auto lineVariant = data(Qt::UserRole);
             // either of these mean we tried to obtain line already
-            if (line.isValid() || rootItem->data(RangeData::KindRole).toBool()) {
-                return QStandardItem::data(role).toString().append(line.toString());
+            if (lineVariant.isValid() || rootItem->data(RangeData::KindRole).toBool()) {
+                return QStandardItem::data(role).toString().append(lineVariant.toString());
             }
 
             KTextEditor::Document *doc = nullptr;
@@ -1443,7 +1462,7 @@ public:
         }
     }
 
-    Q_SIGNAL void ctrlClickDefRecieved(const RangeItem &range);
+    Q_SIGNAL void ctrlClickDefReceieved(const RangeItem &range);
 
     Q_SLOT void onCtrlMouseMove(const RangeItem &range)
     {
@@ -1509,7 +1528,7 @@ public:
                 return;
             } else {
                 const auto item = locationToRangeItem(defs.at(0));
-                Q_EMIT this->ctrlClickDefRecieved(item);
+                Q_EMIT this->ctrlClickDefReceieved(item);
                 return;
             }
         };
@@ -1949,9 +1968,9 @@ public:
         if (!server) {
             return;
         }
-        auto d = new GotoSymbolHUDDialog(m_mainWindow, server);
-        d->raise();
-        d->show();
+        auto dialog = new GotoSymbolHUDDialog(m_mainWindow, server);
+        dialog->raise();
+        dialog->show();
     }
 
     static QStandardItem *getItem(const QStandardItemModel &model, const QUrl &url)
@@ -1968,8 +1987,7 @@ public:
     {
         QStandardItem *targetItem = nullptr;
         if (topItem) {
-            int count = topItem->rowCount();
-            int first = 0, last = count;
+            const int count = topItem->rowCount();
             // let's not run wild on a linear search in a flood of diagnostics
             if (count > 50) {
                 // instead, let's *assume* sorted and use binary search to get closer
@@ -1994,6 +2012,7 @@ public:
                     }
                 }
             }
+            int first = 0, last = count;
             for (int i = first; i < last; ++i) {
                 auto item = topItem->child(i);
                 if (!(item->flags() & Qt::ItemIsEnabled)) {
@@ -2239,8 +2258,8 @@ public:
             }
             // only consider basename (full path may have been custom specified)
             auto lspServer = QFileInfo(server->cmdline().front()).fileName();
-            isClangd = lspServer == QStringLiteral("clangd");
-            isRustAnalyzer = lspServer == QStringLiteral("rust-analyzer");
+            isClangd = lspServer == QLatin1String("clangd");
+            isRustAnalyzer = lspServer == QLatin1String("rust-analyzer");
 
             const bool semHighlightingEnabled = m_plugin->m_semanticHighlighting;
             if (semHighlightingEnabled) {
