@@ -96,9 +96,9 @@ static QStringList indicationDataToStringList(const QJsonValue &indicationData)
     return {};
 }
 
-static LSPClientServer::TriggerCharactersOverride parseTriggerOverride(const QJsonValue &json)
+static TriggerCharactersOverride parseTriggerOverride(const QJsonValue &json)
 {
-    LSPClientServer::TriggerCharactersOverride adjust;
+    TriggerCharactersOverride adjust;
     if (json.isObject()) {
         auto ob = json.toObject();
         for (const auto &c : ob.value(QStringLiteral("exclude")).toString()) {
@@ -253,11 +253,13 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     // variable to avoid warning more than once
     QSet<QString> m_failedToFindServers;
 
+    bool m_suspendEnabled = false;
+
 public:
-    LSPClientServerManagerImpl(LSPClientPlugin *plugin)
-        : m_plugin(plugin)
+    LSPClientServerManagerImpl(LSPClientPlugin *lspClientPlugin)
+        : m_plugin(lspClientPlugin)
     {
-        connect(plugin, &LSPClientPlugin::update, this, &self_type::updateServerConfig);
+        connect(lspClientPlugin, &LSPClientPlugin::update, this, &self_type::updateServerConfig);
         QTimer::singleShot(100, this, &self_type::updateServerConfig);
 
         // stay tuned on project situation
@@ -298,7 +300,7 @@ public:
          * So we are left with a minor sleep compromise ...
          */
 
-        int count = 0;
+        int runningCount = 0;
         for (const auto &el : std::as_const(m_servers)) {
             for (const auto &si : el) {
                 auto &s = si.server;
@@ -307,12 +309,12 @@ public:
                 }
                 disconnect(s.get(), nullptr, this, nullptr);
                 if (s->state() != LSPClientServer::State::None) {
-                    ++count;
+                    ++runningCount;
                     s->stop(-1, -1);
                 }
             }
         }
-        if (count) {
+        if (runningCount > 0) {
             QThread::msleep(500);
         } else {
             return;
@@ -408,6 +410,10 @@ public:
 
     std::shared_ptr<LSPClientServer> findServer(KTextEditor::View *view, bool updatedoc = true) override
     {
+        if (m_suspendEnabled) {
+            return nullptr;
+        }
+
         if (!view) {
             return nullptr;
         }
@@ -477,6 +483,11 @@ public:
         return result;
     }
 
+    void setSuspendEnabled(bool suspendEnabled) override
+    {
+        m_suspendEnabled = suspendEnabled;
+    }
+
 private:
     void showMessage(const QString &msg, KTextEditor::Message::MessageType level)
     {
@@ -872,7 +883,7 @@ private:
                 if (!m_failedToFindServers.contains(cmdline[0])) {
                     m_failedToFindServers.insert(cmdline[0]);
                     // we didn't find the server binary at all!
-                    QString message = i18n("Failed to find server binary: %1", cmdline[0]);
+                    QString message = i18n("Failed to find server binary '%1' for language '%2'", cmdline[0], langId);
                     const auto url = serverConfig.value(QStringLiteral("url")).toString();
                     if (!url.isEmpty()) {
                         message += QStringLiteral("\n") + i18n("Please check your PATH for the binary");
@@ -897,7 +908,7 @@ private:
             // since that is what a server is more aware of
             // and should support if it declares workspace folder capable
             // (as opposed to the new initialization property)
-            LSPClientServer::FoldersType folders;
+            FoldersType folders;
             if (useWorkspace) {
                 folders = QList<LSPWorkspaceFolder>();
             }
@@ -906,7 +917,7 @@ private:
             // extract some more additional config
             auto completionOverride = parseTriggerOverride(serverConfig.value(QStringLiteral("completionTriggerCharacters")));
             auto signatureOverride = parseTriggerOverride(serverConfig.value(QStringLiteral("signatureTriggerCharacters")));
-            decltype(LSPClientServer::ExtraServerConfig::environment) env;
+            decltype(ExtraServerConfig::environment) env;
 
             if (!execPrefix.isEmpty()) {
                 if (!pathMapping) {
@@ -923,16 +934,21 @@ private:
             Utils::setupFlatpakPathMapping(pathMapping);
 
             // request server and setup
-            server.reset(new LSPClientServer(
-                cmdline,
-                root,
-                realLangId,
-                serverConfig.value(QStringLiteral("initializationOptions")),
-                {.folders = folders, .caps = caps, .completion = completionOverride, .signature = signatureOverride, .map = pathMapping, .environment = env}));
+            server.reset(new LSPClientServer(cmdline,
+                                             root,
+                                             realLangId,
+                                             serverConfig.value(QStringLiteral("initializationOptions")),
+                                             {.folders = folders,
+                                              .caps = caps,
+                                              .completion = completionOverride,
+                                              .signature = signatureOverride,
+                                              .map = pathMapping,
+                                              .allowExperimental = serverConfig.value(QStringLiteral("allowExperimental")).toBool(),
+                                              .environment = env}));
             connect(server.get(), &LSPClientServer::stateChanged, this, &self_type::onStateChanged, Qt::UniqueConnection);
             connect(server.get(), &LSPClientServer::extraData, this, &self_type::onExtraData, Qt::UniqueConnection);
             if (!server->start(m_plugin->m_debugMode)) {
-                QString message = i18n("Failed to start server: %1", cmdline.join(QLatin1Char(' ')));
+                QString message = i18n("Failed to start server '%1' for language '%2'", cmdline.join(QLatin1Char(' ')), langId);
                 const auto url = serverConfig.value(QStringLiteral("url")).toString();
                 if (!url.isEmpty()) {
                     message += QStringLiteral("\n") + i18n("Please check your PATH for the binary");
@@ -959,7 +975,9 @@ private:
     {
         // default configuration, compiled into plugin resource, reading can't fail
         QFile defaultConfigFile(QStringLiteral(":/lspclient/settings.json"));
-        defaultConfigFile.open(QIODevice::ReadOnly);
+        if (!defaultConfigFile.open(QIODevice::ReadOnly)) {
+            Q_UNREACHABLE();
+        }
         Q_ASSERT(defaultConfigFile.isOpen());
         m_serverConfig = QJsonDocument::fromJson(defaultConfigFile.readAll()).object();
 
