@@ -6,6 +6,7 @@
 #include "rainbowparens_plugin.h"
 
 #include <QGradient>
+#include <QGroupBox>
 #include <QIcon>
 #include <QLabel>
 #include <QPainter>
@@ -19,8 +20,6 @@
 #include <KTextEditor/Document>
 #include <KTextEditor/View>
 
-constexpr int numberOfColors = 5;
-
 K_PLUGIN_FACTORY_WITH_JSON(RainbowParenPluginFactory, "rainbowparens_plugin.json", registerPlugin<RainbowParenPlugin>();)
 
 RainbowParenPlugin::RainbowParenPlugin(QObject *parent)
@@ -39,24 +38,24 @@ KTextEditor::ConfigPage *RainbowParenPlugin::configPage(int number, QWidget *par
 
 void RainbowParenPlugin::readConfig()
 {
-    if (attrs.empty()) {
-        attrs.resize(5);
-        for (auto &attr : attrs) {
+    if (m_attrs.empty()) {
+        m_attrs.resize(5);
+        for (auto &attr : m_attrs) {
             attr = new KTextEditor::Attribute;
         }
     }
 
     KConfigGroup config(KSharedConfig::openConfig(), QStringLiteral("ColoredBrackets"));
     QColor br = config.readEntry("color1", QStringLiteral("#1275ef"));
-    attrs[0]->setForeground(br);
+    m_attrs[0]->setForeground(br);
     br = config.readEntry("color2", QStringLiteral("#f83c1f"));
-    attrs[1]->setForeground(br);
+    m_attrs[1]->setForeground(br);
     br = config.readEntry("color3", QStringLiteral("#9dba1e"));
-    attrs[2]->setForeground(br);
+    m_attrs[2]->setForeground(br);
     br = config.readEntry("color4", QStringLiteral("#e219e2"));
-    attrs[3]->setForeground(br);
+    m_attrs[3]->setForeground(br);
     br = config.readEntry("color5", QStringLiteral("#37d21c"));
-    attrs[4]->setForeground(br);
+    m_attrs[4]->setForeground(br);
 }
 
 QObject *RainbowParenPlugin::createView(KTextEditor::MainWindow *mainWindow)
@@ -72,14 +71,14 @@ RainbowParenPluginView::RainbowParenPluginView(RainbowParenPlugin *plugin, KText
     connect(mainWin, &KTextEditor::MainWindow::viewChanged, this, &RainbowParenPluginView::viewChanged);
     QTimer::singleShot(50, this, [this] {
         if (auto *view = m_mainWindow->activeView()) {
-            rehighlight(view);
+            m_lastUserColor = rehighlight(view, m_ranges, m_lastUserColor, m_plugin->colorsList());
         }
     });
     m_rehighlightTimer.setInterval(200);
     m_rehighlightTimer.setSingleShot(true);
     m_rehighlightTimer.callOnTimeout(this, [this] {
         if (m_activeView) {
-            rehighlight(m_activeView);
+            m_lastUserColor = rehighlight(m_activeView, m_ranges, m_lastUserColor, m_plugin->colorsList());
         }
     });
 }
@@ -122,10 +121,10 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
         // loose the bracket coloring
         SavedRanges saved;
         saved.doc = doc;
-        saved.ranges = std::move(ranges);
-        savedRanges.push_back(std::move(saved));
-        if (savedRanges.size() > 4) {
-            savedRanges.erase(savedRanges.begin());
+        saved.ranges = std::move(m_ranges);
+        m_savedRanges.push_back(std::move(saved));
+        if (m_savedRanges.size() > 4) {
+            m_savedRanges.erase(m_savedRanges.begin());
         }
 
         // clear ranges for this doc if it gets closed
@@ -139,11 +138,11 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
                 Qt::UniqueConnection);
     }
 
-    ranges.clear();
+    m_ranges.clear();
     m_activeView = view;
 
     // get any existing ranges for this view
-    getSavedRangesForDoc(savedRanges, ranges, m_activeView->document());
+    getSavedRangesForDoc(m_savedRanges, m_ranges, m_activeView->document());
 
     connect(view, &KTextEditor::View::verticalScrollPositionChanged, this, &RainbowParenPluginView::onScrollChanged, Qt::UniqueConnection);
 
@@ -155,7 +154,7 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
     connect(doc, &KTextEditor::Document::textInserted, this, &RainbowParenPluginView::onTextInserted, Qt::UniqueConnection);
     connect(doc, &KTextEditor::Document::textRemoved, this, &RainbowParenPluginView::onTextRemoved, Qt::UniqueConnection);
 
-    rehighlight(m_activeView);
+    m_lastUserColor = rehighlight(m_activeView, m_ranges, m_lastUserColor, m_plugin->colorsList());
 }
 
 static void onTextChanged(RainbowParenPluginView *p, const QString &text)
@@ -201,16 +200,16 @@ void RainbowParenPluginView::requestRehighlight(int delay)
 
 void RainbowParenPluginView::clearRanges(KTextEditor::Document *)
 {
-    ranges.clear();
+    m_ranges.clear();
 }
 
 void RainbowParenPluginView::clearSavedRangesForDoc(KTextEditor::Document *doc)
 {
-    auto it = std::find_if(savedRanges.begin(), savedRanges.end(), [doc](const RainbowParenPluginView::SavedRanges &r) {
+    auto it = std::find_if(m_savedRanges.begin(), m_savedRanges.end(), [doc](const RainbowParenPluginView::SavedRanges &r) {
         return r.doc == doc;
     });
-    if (it != savedRanges.end()) {
-        savedRanges.erase(it);
+    if (it != m_savedRanges.end()) {
+        m_savedRanges.erase(it);
     }
 }
 
@@ -269,14 +268,17 @@ static ColoredBracketPair existingColoredBracketForPos(std::vector<ColoredBracke
  * This allows us to do a lot less work and still be able to do coloring
  * of brackets.
  */
-void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
+size_t rehighlight(KTextEditor::View *view,
+                   std::vector<std::unique_ptr<KTextEditor::MovingRange>> &ranges,
+                   size_t lastUserColorIdx,
+                   const std::vector<KTextEditor::Attribute::Ptr> &attrs)
 {
     // we only care about lines that are in the viewport
     int start = view->firstDisplayedLine();
     int end = view->lastDisplayedLine();
     if (end < start) {
         qWarning("RainbowParenPluginView: Unexpected end < start");
-        return;
+        return lastUserColorIdx;
     }
 
     // if the lines are folded we can get really big range
@@ -284,37 +286,16 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
         end = start + 800;
     }
 
-    /** The brackets that we support for now **/
-    constexpr int totalBracketTypes = 3;
-    static constexpr std::array<QChar, totalBracketTypes> opens = {QLatin1Char('{'), QLatin1Char('('), QLatin1Char('[')};
-    static constexpr std::array<QChar, totalBracketTypes> closes = {QLatin1Char('}'), QLatin1Char(')'), QLatin1Char(']')};
-
     // Check if @p c matches any opener
-    auto matchOpen = [](QChar c) {
-        for (auto o : opens) {
-            if (o == c) {
-                return true;
-            }
-        }
-        return false;
+    auto isBracketOpen = [](QChar c) {
+        return c == u'{' || c == u'[' || c == u'(';
     };
-    // Check if @p c matches any closer
-    auto matchClose = [](QChar c) {
-        for (auto o : closes) {
-            if (o == c) {
-                return true;
-            }
-        }
-        return false;
+    // Check if @p is a bracket closer for @p opener
+    auto matchesCloser = [](QChar in, QChar opener) {
+        return (opener == u'{' && in == u'}') || (opener == u'[' && in == u']') || (opener == u'(' && in == u')');
     };
-    // Check at which index @p c is in openers
-    auto indexOfOpener = [](QChar c) {
-        for (int i = 0; i < (int)opens.size(); ++i) {
-            if (c == opens[i]) {
-                return i;
-            }
-        }
-        return -1;
+    auto createOneColumnRange = [](KTextEditor::Cursor c) {
+        return KTextEditor::Range(c.line(), c.column(), c.line(), c.column() + 1);
     };
 
     // A struct representing an opening bracket
@@ -325,11 +306,6 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
 
     // A struct representing a bracket pair (open, close)
     struct BracketPair {
-        BracketPair(KTextEditor::Cursor oo, KTextEditor::Cursor cc)
-            : opener(oo)
-            , closer(cc)
-        {
-        }
         KTextEditor::Cursor opener;
         KTextEditor::Cursor closer;
     };
@@ -348,27 +324,19 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
                 continue;
             }
 
-            if (matchOpen(line.at(c))) {
-                Opener o;
-                o.bracketChar = line.at(c);
-                o.pos = {l, c};
-                bracketStack.push_back(o);
-            } else if (matchClose(line.at(c))) {
-                if (!bracketStack.empty()) {
-                    auto opener = bracketStack.back();
-                    int idx = indexOfOpener(opener.bracketChar);
-                    if (idx == -1) {
-                        continue;
-                    }
-                    QChar closerChar = closes[idx];
-
-                    if (closerChar != line.at(c)) {
-                        continue;
-                    }
-
-                    parens.push_back({opener.pos, {l, c}});
-                    bracketStack.pop_back();
+            if (isBracketOpen(line.at(c))) {
+                bracketStack.push_back(Opener{
+                    .bracketChar = line.at(c),
+                    .pos = {l, c},
+                });
+            } else if (!bracketStack.empty() && matchesCloser(line.at(c), bracketStack.back().bracketChar)) {
+                auto openerPos = bracketStack.back().pos;
+                bracketStack.pop_back();
+                // no '()' or '{}' highlighting
+                if (openerPos.line() == l && openerPos.column() + 1 == c) {
+                    continue;
                 }
+                parens.push_back({openerPos, {l, c}});
             }
         }
     }
@@ -378,8 +346,9 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
     // and not change all the time
     auto oldRanges = std::move(ranges);
 
-    if (parens.empty())
-        return;
+    if (parens.empty()) {
+        return lastUserColorIdx;
+    }
 
     // sort by start paren
     // Necessary so that we can get alternating colors for brackets
@@ -393,9 +362,8 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
     };
 
     size_t idx = 0;
-    size_t color = m_lastUserColor;
+    size_t color = lastUserColorIdx;
     int lastParenLine = 0;
-    const auto &attrs = m_plugin->colorsList();
     for (auto p : parens) {
         // scope guard to ensure we always update stuff for every iteration
         auto updater = qScopeGuard([&idx, &lastParenLine, p] {
@@ -403,14 +371,6 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
             lastParenLine = p.opener.line();
         });
 
-        auto cur1 = p.opener;
-        cur1.setColumn(cur1.column() + 1);
-
-        // no '()' or '{}' highlighting
-        if (cur1 == p.closer) {
-            continue;
-        }
-
         // open/close brackets on same line?
         if (lastParenLine != p.opener.line() && onSameLine(p.opener, p.closer)) {
             // check next position,
@@ -426,16 +386,14 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
         auto [existingStart, existingEnd] = existingColoredBracketForPos(oldRanges, p.opener, p.closer);
         if (existingStart && existingEnd) {
             // ensure start and end have same attribute and expected range
-            KTextEditor::Range expectedStart = {p.opener, cur1};
+            KTextEditor::Range expectedStart = createOneColumnRange(p.opener);
             if (existingStart->toRange() != expectedStart) {
                 existingStart->setRange(expectedStart, existingEnd->attribute());
             } else if (existingStart->attribute() != existingEnd->attribute()) {
                 existingStart->setAttribute(existingEnd->attribute());
             }
 
-            auto cur2 = p.closer;
-            cur2.setColumn(cur2.column() + 1);
-            KTextEditor::Range expectedEnd = {p.closer, cur2};
+            KTextEditor::Range expectedEnd = createOneColumnRange(p.closer);
             if (existingEnd->toRange() != expectedEnd) {
                 existingEnd->setRange(expectedEnd);
             }
@@ -450,12 +408,10 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
             continue;
         }
 
-        std::unique_ptr<KTextEditor::MovingRange> r(doc->newMovingRange({p.opener, cur1}));
+        std::unique_ptr<KTextEditor::MovingRange> r(doc->newMovingRange(createOneColumnRange(p.opener)));
         r->setAttribute(attrs[color % numberOfColors]);
 
-        auto cur2 = p.closer;
-        cur2.setColumn(cur2.column() + 1);
-        std::unique_ptr<KTextEditor::MovingRange> r2(doc->newMovingRange({p.closer, cur2}));
+        std::unique_ptr<KTextEditor::MovingRange> r2(doc->newMovingRange(createOneColumnRange(p.closer)));
         r2->setAttribute(attrs[color % numberOfColors]);
 
         ranges.push_back(std::move(r));
@@ -463,8 +419,9 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
 
         color++;
     }
-    m_lastUserColor = color;
+
     oldRanges.clear();
+    return color;
 }
 
 RainbowParenConfigPage::RainbowParenConfigPage(QWidget *parent, RainbowParenPlugin *plugin)
@@ -472,22 +429,18 @@ RainbowParenConfigPage::RainbowParenConfigPage(QWidget *parent, RainbowParenPlug
     , m_plugin(plugin)
 {
     auto layout = new QVBoxLayout(this);
-    layout->setContentsMargins({});
 
-    auto label = new QLabel(this);
-    label->setText(i18n("Choose colors that will be used for bracket coloring:"));
-    label->setWordWrap(true);
-    layout->addWidget(label);
+    auto gb = new QGroupBox(this);
+    gb->setTitle(i18nc("group title", "Bracket Colors"));
+    auto gbLayout = new QVBoxLayout(gb);
 
     for (auto &btn : m_btns) {
-        auto hl = new QHBoxLayout;
-        hl->addWidget(&btn);
-        hl->addStretch();
-        hl->setContentsMargins({});
-        layout->addLayout(hl);
-        btn.setMinimumWidth(150);
+        gbLayout->addWidget(&btn);
+        btn.setMaximumWidth(150);
         connect(&btn, &KColorButton::changed, this, &RainbowParenConfigPage::changed);
     }
+
+    layout->addWidget(gb);
     layout->addStretch();
 
     reset();
