@@ -5,17 +5,20 @@
 #include "ui_template.h"
 
 #include <KLocalizedString>
+#include <KMessageBox>
 
-#include <QDebug>
 #include <QDir>
+#include <QDirIterator>
 #include <QFileDialog>
 #include <QFileInfo>
+#include <QInputDialog>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonParseError>
 #include <QStandardPaths>
 #include <QTemporaryDir>
+#include <QtCore/qlogging.h>
 
 using namespace Qt::Literals::StringLiterals;
 
@@ -117,10 +120,21 @@ Template::Template(QWidget *parent)
     ui->setupUi(this);
     m_createButton = new QPushButton(i18n("Create"));
     m_createButton->setEnabled(false);
-    ui->u_buttonBox->addButton(m_createButton, QDialogButtonBox::AcceptRole);
 
+    ui->u_buttonBox->addButton(m_createButton, QDialogButtonBox::ActionRole);
+    m_createButton->setDefault(true);
+
+    m_exportButton = new QPushButton(i18n("Export"));
+    m_exportButton->setEnabled(false);
+    ui->u_buttonBox->addButton(m_exportButton, QDialogButtonBox::ActionRole);
+
+    QPushButton *importButton = new QPushButton(QIcon::fromTheme(u"document-import"_s), i18n("Import..."), this);
+    ui->u_buttonBox->addButton(importButton, QDialogButtonBox::ActionRole);
+
+    connect(importButton, &QPushButton::clicked, this, &Template::importTemplate);
     connect(m_createButton, &QPushButton::clicked, this, &Template::createFromTemplate);
     connect(ui->u_buttonBox, &QDialogButtonBox::rejected, this, &Template::cancel);
+    connect(m_exportButton, &QPushButton::clicked, this, &Template::exportTemplate);
 
     ui->u_templateTree->setHeaderHidden(true);
     ui->u_templateTree->setModel(&m_selectionModel);
@@ -159,16 +173,16 @@ void Template::addEntries(const QFileInfo &info, const QModelIndex &parent)
     QDir dir(info.absoluteFilePath());
     QStringList files = dir.entryList(QDir::Files | QDir::Hidden);
     if (files.contains(u"template.json"_s)) {
-        std::unique_ptr<TreeData> data = std::make_unique<TreeData>();
-        data->path = parent.data(TreeData::PathRole).toString();
-        data->configFile = u"template.json"_s;
-        m_selectionModel.setAbstractData(std::move(data), parent);
+        std::unique_ptr<TreeData> treeData = std::make_unique<TreeData>();
+        treeData->path = parent.data(TreeData::PathRole).toString();
+        treeData->configFile = u"template.json"_s;
+        m_selectionModel.setAbstractData(std::move(treeData), parent);
         return;
     }
     for (const auto &entry : dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot)) {
-        std::unique_ptr<TreeData> data = std::make_unique<TreeData>();
-        data->path = entry.absoluteFilePath();
-        const QModelIndex cIndex = m_selectionModel.addChild(std::move(data), parent);
+        std::unique_ptr<TreeData> treeData = std::make_unique<TreeData>();
+        treeData->path = entry.absoluteFilePath();
+        const QModelIndex cIndex = m_selectionModel.addChild(std::move(treeData), parent);
         addEntries(entry, cIndex);
     }
 }
@@ -343,6 +357,9 @@ void Template::templateIndexChanged(const QModelIndex &newIndex)
     ui->u_detailsTB->setText(QString());
     ui->u_configWidget->setEnabled(false);
 
+    m_exportButton->setEnabled(false);
+    m_createButton->setEnabled(false);
+
     QString path = newIndex.data(TreeData::PathRole).toString();
     QString config = newIndex.data(TreeData::ConfigJsonRole).toString();
 
@@ -358,7 +375,6 @@ void Template::templateIndexChanged(const QModelIndex &newIndex)
 #endif
 
     QByteArray configJson;
-
     ui->u_lowercaseCheckBox->setEnabled(true);
 
     QFile in(path + '/'_L1 + config);
@@ -381,21 +397,24 @@ void Template::templateIndexChanged(const QModelIndex &newIndex)
 
     for (const auto renameVal : replacements) {
         const auto renObj = renameVal.toObject();
-        std::unique_ptr<ConfigData> data = std::make_unique<ConfigData>();
-        data->m_desc = renObj.value(u"description"_s).toString();
-        data->m_placeholder = renObj.value(u"placeholder"_s).toString().toLocal8Bit();
-        data->m_value = renObj.value(u"default"_s).toString().toLocal8Bit();
-        data->m_mustBeLowercase = renObj.value(u"mustBeLowercase"_s).toBool(false);
+        std::unique_ptr<ConfigData> configData = std::make_unique<ConfigData>();
+        configData->m_desc = renObj.value(u"description"_s).toString();
+        configData->m_placeholder = renObj.value(u"placeholder"_s).toString().toLocal8Bit();
+        configData->m_value = renObj.value(u"default"_s).toString().toLocal8Bit();
+        configData->m_mustBeLowercase = renObj.value(u"mustBeLowercase"_s).toBool(false);
         const auto files = renObj.value(u"genratedFiles"_s).toArray();
         for (const auto file : files) {
-            data->m_generatedFiles.append(file.toString());
+            configData->m_generatedFiles.append(file.toString());
         }
-        m_configModel.addChild(std::move(data), QModelIndex());
+        m_configModel.addChild(std::move(configData), QModelIndex());
     }
 
     ui->u_configTreeView->resizeColumnToContents(0);
     ui->u_configTreeView->resizeColumnToContents(1);
     ui->u_configWidget->setEnabled(true);
+
+    m_exportButton->setEnabled(true);
+    checkIfConfigIsReady();
 }
 
 void Template::checkIfConfigIsReady()
@@ -444,10 +463,10 @@ void Template::addAppWizardTemplates()
     }
 
     // create the appWiz root
-    std::unique_ptr<TreeData> data = std::make_unique<TreeData>();
-    data->path = i18n("KAppTemplates");
+    std::unique_ptr<TreeData> rootData = std::make_unique<TreeData>();
+    rootData->path = i18n("KAppTemplates");
     QMap<QString, QModelIndex> indexes;
-    indexes.insert(QString(), m_selectionModel.addChild(std::move(data), QModelIndex()));
+    indexes.insert(QString(), m_selectionModel.addChild(std::move(rootData), QModelIndex()));
 
     for (auto it = m_appWizMap.cbegin(); it != m_appWizMap.cend(); it++) {
         QStringList itemPaths = it.key().split(u'/');
@@ -458,18 +477,18 @@ void Template::addAppWizardTemplates()
             paths.append(part);
             const QString path = paths.join(u'/');
             if (!indexes.contains(path)) {
-                std::unique_ptr<TreeData> data = std::make_unique<TreeData>();
-                data->path = path;
-                index = m_selectionModel.addChild(std::move(data), index);
+                std::unique_ptr<TreeData> treeData = std::make_unique<TreeData>();
+                treeData->path = path;
+                index = m_selectionModel.addChild(std::move(treeData), index);
                 indexes.insert(path, index);
             } else {
                 index = indexes.value(path);
             }
         }
-        std::unique_ptr<TreeData> data = std::make_unique<TreeData>();
-        data->path = it.key();
-        data->configFile = it->packagePath;
-        m_selectionModel.addChild(std::move(data), index);
+        std::unique_ptr<TreeData> treeData = std::make_unique<TreeData>();
+        treeData->path = it.key();
+        treeData->configFile = it->packagePath;
+        m_selectionModel.addChild(std::move(treeData), index);
     }
 }
 
@@ -481,13 +500,13 @@ void Template::appWizardTemplateSelected(const QString &category)
     ui->u_detailsTB->setText(templ.description);
 
     for (const auto &ren : AppWizardReader().replacements()) {
-        std::unique_ptr<ConfigData> data = std::make_unique<ConfigData>();
-        data->m_desc = ren.descr;
-        data->m_placeholder = ren.placeholder;
-        data->m_value = ren.defaultValue;
-        data->m_mustBeLowercase = false;
+        std::unique_ptr<ConfigData> configData = std::make_unique<ConfigData>();
+        configData->m_desc = ren.descr;
+        configData->m_placeholder = ren.placeholder;
+        configData->m_value = ren.defaultValue;
+        configData->m_mustBeLowercase = false;
         // data->m_generatedFiles;
-        m_configModel.addChild(std::move(data), QModelIndex());
+        m_configModel.addChild(std::move(configData), QModelIndex());
     }
 
     ui->u_configTreeView->resizeColumnToContents(0);
@@ -558,7 +577,203 @@ void Template::createFromAppWizardTemplate(const QString &category)
     }
     Q_EMIT done(fileToOpen);
 }
-
 #endif
 
+void Template::exportTemplate()
+{
+    const QModelIndex index = ui->u_templateTree->currentIndex();
+    if (!index.isValid()) {
+        return;
+    }
+
+    const QString sourcePath = index.data(TreeData::PathRole).toString();
+    if (sourcePath.isEmpty()) {
+        KMessageBox::error(this, i18n("Could not retrieve template path."));
+        return;
+    }
+
+    const QString destDir = QFileDialog::getExistingDirectory(this, i18n("Export Template To"), QDir::homePath());
+    if (destDir.isEmpty()) {
+        return;
+    }
+
+    const QString fileName = QFileInfo(sourcePath).fileName();
+    const QString finalDest = QDir(destDir).filePath(fileName);
+
+    if (QFileInfo::exists(finalDest)) {
+        QFileInfo destInfo(finalDest);
+        QString typeStr;
+
+        if (destInfo.isDir()) {
+            typeStr = i18n("folder");
+        } else {
+            typeStr = i18n("file");
+        }
+
+        KMessageBox::error(this, i18n("The %1 '%2' already exists in the destination.\nExport failed.", typeStr, fileName), i18n("Export Failed"));
+
+        return;
+    }
+
+    QDir().mkpath(finalDest);
+    bool success = copyFolder(sourcePath, finalDest, ReplaceMap(), ReplaceMap(), QStringList());
+
+    if (success) {
+        KMessageBox::information(this, i18n("Template exported successfully to:\n%1", finalDest));
+    } else {
+        KMessageBox::error(this, i18n("Failed to export template."));
+    }
+}
+
+void Template::importTemplate()
+{
+    const QString srcDir = QFileDialog::getExistingDirectory(this,
+                                                             i18n("Select Template Folder to Import"),
+                                                             QDir::homePath(),
+                                                             QFileDialog::ShowDirsOnly | QFileDialog::ReadOnly | QFileDialog::DontUseNativeDialog);
+    if (srcDir.isEmpty()) {
+        return;
+    }
+
+    QDir source(srcDir);
+
+    if (!source.exists(u"template.json"_s)) {
+        KMessageBox::error(this, i18n("The selected folder does not contain a valid template.json file."));
+        return;
+    }
+
+    QFile jsonFile(source.filePath(u"template.json"_s));
+    if (!jsonFile.open(QIODevice::ReadOnly)) {
+        KMessageBox::error(this, i18n("Could not open template.json."));
+        return;
+    }
+
+    QJsonParseError error;
+    QJsonDocument doc = QJsonDocument::fromJson(jsonFile.readAll(), &error);
+    jsonFile.close();
+
+    if (error.error != QJsonParseError::NoError) {
+        KMessageBox::error(this, i18n("Invalid JSON in template.json: %1", error.errorString()));
+        return;
+    }
+
+    QJsonObject root = doc.object();
+
+    if (root.contains(u"fileToOpen"_s)) {
+        const QString fileToOpen = root.value(u"fileToOpen"_s).toString();
+        if (!source.exists(fileToOpen)) {
+            KMessageBox::error(this, i18n("Broken Template: The fileToOpen '%1' does not exist in the folder.", fileToOpen));
+            return;
+        }
+    }
+
+    if (!root.contains(u"replacements"_s)) {
+        KMessageBox::error(this, i18n("Invalid Template: Missing mandatory 'replacements' tag in template.json."));
+        return;
+    }
+
+    QSet<QString> unfoundPlaceholders;
+    const QJsonArray replacements = root.value(u"replacements"_s).toArray();
+
+    for (const auto &item : replacements) {
+        if (!item.isObject()) {
+            KMessageBox::error(this, i18n("Invalid Template: 'replacements' array contains invalid data."));
+            return;
+        }
+
+        const QJsonObject replacement = item.toObject();
+
+        if (!replacement.contains(u"description"_s) || !replacement.contains(u"placeholder"_s) || !replacement.contains(u"default"_s)) {
+            KMessageBox::error(this, i18n("Invalid Template: A replacement object is missing required tags (description, placeholder, or default)."));
+            return;
+        }
+
+        const QString placeholder = replacement.value(u"placeholder"_s).toString();
+
+        if (placeholder.isEmpty()) {
+            KMessageBox::error(this, i18n("Invalid Template: A placeholder string cannot be empty."));
+            return;
+        }
+
+        unfoundPlaceholders.insert(placeholder);
+    }
+
+    if (!unfoundPlaceholders.isEmpty()) {
+        QDirIterator it(srcDir, QDir::Files | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
+        while (it.hasNext() && !unfoundPlaceholders.isEmpty()) {
+            QString filePath = it.next();
+            if (QFileInfo(filePath).fileName() == u"template.json"_s) {
+                continue;
+            }
+
+            QFile f(filePath);
+            if (f.open(QIODevice::ReadOnly | QIODevice::Text)) {
+                QString content = QString::fromUtf8(f.readAll());
+                for (auto i = unfoundPlaceholders.begin(); i != unfoundPlaceholders.end();) {
+                    if (content.contains(*i)) {
+                        i = unfoundPlaceholders.erase(i);
+                    } else {
+                        ++i;
+                    }
+                }
+            }
+        }
+
+        if (!unfoundPlaceholders.isEmpty()) {
+            QString missing = QStringList(unfoundPlaceholders.values()).join(u", ");
+            KMessageBox::error(this, i18n("Broken Template: Placeholders defined but not used:\n%1", missing));
+            return;
+        }
+    }
+
+    bool ok = false;
+    QString category = QInputDialog::getText(this,
+                                             i18n("Import Template"),
+                                             i18n("Enter the category (tree-path) for this template (e.g., Files/Qt/FooBar):"),
+                                             QLineEdit::Normal,
+                                             i18n("Imported"),
+                                             &ok);
+
+    if (!ok || category.isEmpty()) {
+        return;
+    }
+
+    const QString localTemplates = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + u"/templates"_s;
+    const QString dirName = source.dirName();
+    const QString destPath = QDir(localTemplates).filePath(category + QLatin1Char('/') + dirName);
+
+    if (QFileInfo::exists(destPath)) {
+        const int ret =
+            KMessageBox::warningContinueCancel(this,
+                                               i18n("A template '%1' already exists in category '%2'.\nDo you want to overwrite it?", dirName, category),
+                                               i18n("Overwrite Template"),
+                                               KStandardGuiItem::overwrite(),
+                                               KStandardGuiItem::cancel(),
+                                               u"overwrite_imported_template_warning"_s);
+
+        if (ret != KMessageBox::Continue) {
+            return;
+        }
+
+        QDir(destPath).removeRecursively();
+    }
+
+    QDir().mkpath(destPath);
+    bool success = copyFolder(srcDir, destPath, ReplaceMap(), ReplaceMap(), QStringList());
+
+    if (success) {
+        m_selectionModel.clear();
+
+        addTemplateRoot(localTemplates);
+        addTemplateRoot(u":/templates"_s);
+
+#ifdef BUILD_APPWIZARD
+        addAppWizardTemplates();
+#endif
+        KMessageBox::information(this, i18n("Template imported successfully!"));
+    } else {
+        KMessageBox::error(this, i18n("Failed to import template."));
+    }
+}
+
 #include "moc_template.cpp"
